/*
  Timeutils.c
  This module provides (hopefully quite portable) timing routines.
  
  Call time_now() to return a milliseconds count
  Call measure_clock_speed(int n_avg) to calculate the CPU's clock speed, where
                              n = number of 1s averages to do.
                              Suggested: n_avg = 2
*/

#include <windows.h>
#include <time.h>

// Local includes
// This following include provides a high performance Intel timing function
// based on the CPU clock. The clock speed needs to be determined to
// use the function.
#include "intelhpt.h" 

#define MAKE_TIME(h,m,s,c)  (long) (h) * 1000000L +                          \
                            (long) (m) * 10000L +                            \
                            (long) (s) * 1000 + (c)

/* Function time_now returns a value in milliseconds indicating current time */
long time_now (void)
{
#if (defined (__UTYPE_FREEBSD__))
    return (timer_to_time (time (NULL)));

#elif (defined (__UNIX__) || defined (__VMS_XOPEN))
    /*  The BSD gettimeofday function returns seconds and microseconds       */
    struct timeval
        time_struct;

    gettimeofday (&time_struct, 0);
    return (timer_to_time (time_struct.tv_sec)
                         + time_struct.tv_usec / 1000);

#elif (defined (WIN32))
    /*  The Win32 GetLocalTime function returns just what we want            */
    SYSTEMTIME time_struct;

    GetLocalTime (&time_struct);
    
    return (MAKE_TIME (time_struct.wHour,
                       time_struct.wMinute,
                       time_struct.wSecond,
                       time_struct.wMilliseconds
                       ));

#else
    /*  Otherwise, just get the time without milliseconds                    */
    return (timer_to_time (time (NULL)));
#endif
}

unsigned long long measure_clock_speed ()
{
    time_t sec0 = time_now();
    time_t end0 = sec0 + 1000;
    time_t t;
    rdtsc_time cnt0 = rdtsc();
    do {
        t = time_now();
    } while (t < end0);
    rdtsc_time cnt1 = rdtsc();
    // display the CPU speed
    unsigned long long clock_ticks;
    if (cnt0>cnt1) clock_ticks = cnt0 - cnt1;
       else clock_ticks = cnt1 - cnt0;
    return clock_ticks;
}

int avg_cpu_speed (int n_avg)
{
    int i;
    printf("START MEASURING CPU SPEED.\n");
    unsigned long long avg_clockspeed = 0;
    for (i=0; i<n_avg+1; i++) {
      unsigned long long clockspeed = measure_clock_speed();
      double freqMHz = (double)clockspeed;
      printf("Trial %d CPU clock ticks: %llu\n",i,clockspeed);
      if (i>0) avg_clockspeed += clockspeed;
      }
    avg_clockspeed = (unsigned long long)avg_clockspeed / n_avg;
    printf("Average clock speed = %llu clocks/sec\n",avg_clockspeed);
    unsigned mhz_clockspeed = (unsigned long long)avg_clockspeed/1000;
    mhz_clockspeed = round((float)mhz_clockspeed / 1000.0);
    printf("Average clock speed = %u MHz\n",mhz_clockspeed);
}

unsigned long long int CPU_ticks ()
{
         return rdtsc();
}

void show_rdtsc_stats ( unsigned long long int time0, unsigned long long int time1 )
{
    printf ("\tTIMING STATISTCS\n");
    printf ("\thardware register starting count  %lu cycles\n", time0);
    printf ("\thardware register ending count  %lu cycles\n", time1);
    printf ("\tcycles:                 %lu \n", time1-time0);
    double seconds = ((float)(time1-time0))/MY_CPU_CLOCK;
    printf ("\tSeconds:                %e \n", seconds);
    // if it took less than a second, print out the value in ms or us...
    if (seconds < 1.0) {
       if (seconds > 0.00099)
       printf ("\tMilliseconds:           %f \n", ((float)1000.0*(time1-time0))/MY_CPU_CLOCK);
       else
       printf ("\tMicroseconds:           %f \n", ((float)1000000.0*(time1-time0))/MY_CPU_CLOCK);
       }
}

